/** \file
 * Header for the CAtlAtmosphere interface
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Cong Loc Kieu, Richard Young
 * \version 1.0
 * \date 04/02/2002
 */

#ifndef CAtlAtmosphere_h
#define CAtlAtmosphere_h


// Basic class.
#ifndef CAtlColor_h
	#include "CAtlColor.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


/// Useful in defining atmospheric effects (fog, smoke, clouds, etc.)
/** \interface CAtlAtmosphere 
 *	Atmosphere specifies the fog parameters.
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Cong Loc Kieu, Richard Young
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlAtmosphere
{
public:
	/// Control if the atmosphere is used.
	virtual bool					getFlag() const = 0;
   /// Control if the atmosphere is used.
	virtual void					setFlag(bool flag) = 0;

	/// Control the color of the fog.
	virtual const CAtlColor&		getColor() const = 0;
   /// Control the color of the fog.
	virtual void					setColor(const CAtlColor& color) = 0;

	/** Control fog density above camera level.
	    Value must be between 0.0 and 1.0.  */
	virtual double					getHighDensity() const = 0;
	/** Control fog density above camera level.
	    Value must be between 0.0 and 1.0.  */
	virtual void					setHighDensity(double density) = 0;

	/** Control fog density at camera level.
	    Value must be between 0.0 and 1.0.  */
	virtual double					getMediumDensity() const = 0;
	/** Control fog density at camera level.
	    Value must be between 0.0 and 1.0.  */
	virtual void					setMediumDensity(double density) = 0;

	/** Control fog density below camera level.
	    Value must be between 0.0 and 1.0.  */
	virtual double					getLowDensity() const = 0;
	/** Control fog density below camera level.
	    Value must be between 0.0 and 1.0.  */
	virtual void					setLowDensity(double density) = 0;

	/** Simple scale factor.
	    This value must be positive.        */
	virtual double					getScale() const = 0;
	/** Simple scale factor.
	    This value must be positive.        */
	virtual void					setScale(double scale) = 0;

	/** Control the distance where the fog starts.
	    This value must be positive.        */
	virtual double					getMinDistance() const = 0;
	/** Control the distance where the fog starts.
	    This value must be positive.        */
	virtual void					setMinDistance(double distance) = 0;

	/** Control the distance where the fog is completely impenetrable.
	    This value must be positive. */
	virtual double					getMaxDistance() const = 0;
	/** Control the distance where the fog is completely impenetrable.
	    This value must be positive. */
	virtual void					setMaxDistance(double distance) = 0;

	/** Control a turbulence factor for the fog.
	    Value should be positive to be considered. */
	virtual double					getNoise() const = 0;
	/** Control a turbulence factor for the fog.
	    Value should be positive to be considered. */
	virtual void					setNoise(double noise) = 0;

// End of API 1
	/// Put an XML representation of the atmosphere inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the atmosphere with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(XMLNodeRef inFatherNode) = 0;

	/// Control if we must use the default atmosphere.
	virtual bool					getUseDefault() const = 0;
	/// Control if we must use the default atmosphere.
	virtual void					setUseDefault(bool inUse) = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual							~CAtlAtmosphere() {}
};


} // namespace  AvAtl

#endif
