/** \file
 * Header for the CAtlBackground interface
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Cong Loc Kieu, Richard Young
 * \version 1.0
 * \date 04/02/2002
 */

#ifndef CAtlBackground_h
#define CAtlBackground_h


// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef CAtlColor_h
	#include "CAtlColor.h"
#endif
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


// We make our enums int, so you don't have to change your project settings.
#ifdef __MWERKS__
#pragma enumsalwaysint on
#endif


/// Define the content of the rendering background (color, gradient, image, etc.)
/** \interface CAtlBackground 
 *	Background rendering is used where there are no objects.
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Cong Loc Kieu, Richard Young
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlBackground
{
public:
	/// Type of Background
	enum EBgImageType
	{
	    eBgNoImage,
	    eBgStretchedImage,
	    eBg3DImage,
	    eBgMotifImage,
	    eBgQTVRCubic
	};

	/// Type of Background
	enum EBgColorType
	{
	    eBgColorNone,			// To don't use alpha channel of the image
	    eBgColorUniform,
	    eBgColorGraduated,
	    eBgColorGraduatedLine
	};

	/// This function have been replaced by getImageType & getColorType
	virtual int						getObsoleteType() const = 0;
	/// This function have been replaced by setImageType & setColorType
	virtual void					setObsoleteType(int inType) = 0;

	/// Control the color used for uniform backgrounds or the first color for the graduated type.
	virtual const CAtlColor&		getColor() const = 0;
	/// Control the color used for uniform backgrounds or the first color for the graduated type.
	virtual void					setColor(const CAtlColor& inColor) = 0;

	/// Control the bottom color of a graduated and a graduated line.
	virtual const CAtlColor&		getSecondColor() const = 0;
	/// Control the bottom color of a graduated and a graduated line.
	virtual void					setSecondColor(const CAtlColor& inColor) = 0;

	/// Control the direction of a graduated or a graduated line
	virtual const CAtlVector&		getUpDirection() const = 0;
	/// Control the direction of a graduated or a graduated line
	virtual void					setUpDirection(const CAtlVector& inUpDir) = 0;

	/// Used for a graduated line.
	/** The greater the value, the smaller the line.
	    Value should be between 10.0 and 100.0. */
	virtual double					getLinePower() const = 0;
	/// Used for a graduated line.
	/** The greater the value, the smaller the line.
	    Value should be between 10.0 and 100.0. */
	virtual void					setLinePower(double inPower) = 0;

	/// The URL of the file used as a picture.
	virtual tURL					getImageURL() const = 0;
	/// The URL of the file used as a picture.
	virtual void					setImageURL(tURL inURL) = 0;

	/// Control the image offset angle for the picture.
	/** Value must be between 0.0 and the width of the image */
	virtual double					obsoleteGet3DPictureOrigin() const = 0;
	/// Control the image offset angle for the picture.
	/** Value must be between 0.0 and the width of the image */
	virtual void					obsoleteSet3DPictureOrigin(double inOrigin) = 0;

// End of API 1
	/// Put an XML representation of the background inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the background with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(XMLNodeRef inFatherNode) = 0;

	/// Control the type of background image.
	virtual EBgImageType			getImageType() const = 0;
	/// Control the type of background image.
	virtual void					setImageType(EBgImageType inType) = 0;

	/// Control the type of background color.
	virtual EBgColorType			getColorType() const = 0;
	/// Control the type of background color.
	virtual void					setColorType(EBgColorType inType) = 0;

	/// Control if we must use the default background.
	virtual bool					getUseDefault() const = 0;
	/// Control if we must use the default background.
	virtual void					setUseDefault(bool inUse) = 0;

	/// Control the image offset angle for picture.
	/** Value must be between 0.0 and 1.0 */
	virtual double					getPictureOrigin() const = 0;
	/// Control the image offset angle for picture.
	/** Value must be between 0.0 and 1.0 */
	virtual void					setPictureOrigin(double inOrigin) = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class.  */
	virtual							~CAtlBackground() {}
};


#ifdef __MWERKS__
#pragma enumsalwaysint reset
#endif


} // namespace  AvAtl

#endif
