/** \file
 * Header for the CAtlCamKey interface
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 30/05/2002
 */

#ifndef CAtlCamKey_h
#define CAtlCamKey_h


// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


// We make our enums int, so you don't have to change your project settings.
#ifdef __MWERKS__
#pragma enumsalwaysint on
#endif


/// Animation timing or camera behavior.
/** \interface CAtlCamKey
 *	The information on timing or camera behavior
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 30/05/2002
 */
class CAtlCamKey
{
public:
	/// Behavior of camera.
	enum ECamKeyType
	{
		eTimingCamKey,
		ePathCamKey,
		eLocalTravCamKey,
		eGlobalTravCamKey,
		eTargetCamKey
	};

	/// Get the camera inclination
	/** Pitch defines a camera incline relative to the horizon.
		Value must be between 0.0 and 360.0. */
	virtual double				getPitch() const = 0;
	/// Set the camera inclination
	/** Pitch defines a camera incline relative to the horizon.
		Value must be between 0.0 and 360.0. */
	virtual void				setPitch(double pitch) = 0;

	/// Get the focal angle.
	/** The smaller the angle, the greater the zoom.
		Value must be between 1.0 and 180.0. */
	virtual double				getFocal() const = 0;
	/// Set the focal angle.
	/** The smaller the angle, the greater the zoom.
		Value must be between 1.0 and 180.0. */
	virtual void				setFocal(double focal) = 0;

	/// Get the behavior of camera.
	virtual ECamKeyType		getType() const = 0;
	/// Set the behavior of camera.
	virtual void				setType(ECamKeyType type) = 0;

	/// Get the fading duration.
	virtual int					getFadingDuration() const = 0;
	/// Set the fading duration.
	virtual void				setFadingDuration(int duration) = 0;

	/// Set the numbre of images for fading.
	virtual int					getNbFadingImages() const = 0;
	/// Set the numbre of images for fading.
	virtual void				setNbFadingImages(int fimages) = 0;

	/// Get the horizontal angle
	/** Used if type == eGlobalTravCamKey
		Value must be between -180.0 and 180.0. */
	virtual double				getHorizontalAngle() const = 0;
	/// Set the horizontal angle
	/** Used if type == eGlobalTravCamKey
		Value must be between -180.0 and 180.0. */
	virtual void				setHorizontalAngle(double angle) = 0;

	/// Get the vertical angle
	/** Used if type == eGlobalTravCamKey
		Value must be between -180.0 and 180.0. */
	virtual double				getVerticalAngle() const = 0;
	/// Set the vertical angle
	/** Used if type == eGlobalTravCamKey
		Value must be between -180.0 and 180.0. */
	virtual void				setVerticalAngle(double angle) = 0;

	/// Get the distance between the view point and the camera.
	/** Used if type == ePathCamKey
		Value must be between 0.0 and 1.0. */
	virtual double				getAnticipation() const = 0;
	/// Set the distance between the view point and the camera.
	/** Used if type == ePathCamKey
		Value must be between 0.0 and 1.0. */
	virtual void				setAnticipation(double) = 0;

	/// Get bezier section
	virtual long				getBezierSection() const = 0;
	/// Set bezier section
	virtual void				setBezierSection(long section) = 0;

	// Get the position in this bezier section.
	/** Value must be between 0.0 and 1.0. */
	virtual double				getTBezier() const = 0;
	// Set the position in this bezier section.
	/** Value must be between 0.0 and 1.0. */
	virtual void				setTBezier(double tbezier) = 0;

	/// Get the bezier section number for the transition point.
	/** Index begins at 0. */
	virtual long				getBezierTransition() const = 0;
	/// Set the bezier section number for the transition point.
	/** Index begins at 0. */
	virtual void				setBezierTransition(long btrans) = 0;

	/// Get the position in this bezier section for the transition point
	/** Value must be between 0.0 and 1.0. */
	virtual double				getTBezierTransition() const = 0;
	/// Set the position in this bezier section for the transition point
	/** Value must be between 0.0 and 1.0. */
	virtual void				setTBezierTransition(double tbtrans) = 0;

	/// Get the target position in 3D space.
	virtual const CAtlVector&	getTarget() const = 0;
	/// Set the target position in 3D space.
	virtual void				setTarget(const CAtlVector& target) = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class.*/
	virtual						~CAtlCamKey() {}
};


#ifdef __MWERKS__
#pragma enumsalwaysint reset
#endif


} // namespace  AvAtl

#endif
