/** \file
 * Header for the CAtlCamera interface
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Cong Loc Kieu, Richard Young
 * \version 1.0
 * \date 04/02/2002
 */

#ifndef CAtlCamera_h
#define CAtlCamera_h


// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif
#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


/// Define the camera to construct a view.
/** \interface CAtlCamera 
 *	Camera information (Observator position, Point of view, focal angle and so on)
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Cong Loc Kieu, Richard Young
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlCamera
{
public:
	/// Return the name of the camera.
	/** Do not delete this string. */
	virtual tUtf8					getName() const = 0;

	/// Set the name of the camera.
	/** \param inName the new camara's name in unicode. */
	virtual void					setName(tUtf8 inName) = 0;

	/// Camera location in the 3D space.
	virtual const CAtlVector&		getPosition() const = 0;
	/// Camera location in the 3D space.
	/** \param inPosition the point where the camera is. */
	virtual void					setPosition(const CAtlVector& inPosition) = 0;

	/// A 3D point the camera looks at.
	virtual const CAtlVector&		getTarget() const = 0;
	/// A 3D point the camera looks at.
	/** \param inTarget the point that the camera looks at. */
	virtual void					setTarget(const CAtlVector& inTarget) = 0;

	/// Is this camera the main view?
	virtual bool					getViewMainFlag() const = 0;
	/// Set this camera as the main view.
	virtual void					setViewMainFlag() = 0;

	/// Is this camera rendered by requesting Render All Cameras?
	/** \retval true the camera is rendered by requesting Render All Cameras.
		 \retval false the camera will not be rendered by requesting Render All Cameras. */
	virtual bool					getViewOnFlag() const = 0;
	/// Set the state to be rendered by requesting Render All Cameras.
	/** \param inState true if the camera must be rendered by requesting Render All Cameras. */
	virtual void					setViewOnFlag(bool inState) = 0;

	/// Roll defines a camera's roll angle relative to the horizon.
	/** \return angle in radians [0.0..2pi]. */
	virtual double					getRoll() const = 0;
	/// Roll defines a camera's roll angle relative to the horizon.
	/** \param inRoll angle in radians [0.0..2pi]. */
	virtual void					setRoll(double inRoll) = 0;

	/// Focal angle. The smaller the angle, the greater the zoom.
	/** Value must be between 1.0 and 180.0. */
	virtual double					getFocal() const = 0;
	/// Focal angle. The smaller the angle, the greater the zoom.
	/** Value must be between 1.0 and 180.0. */
	virtual void					setFocal(double focal) = 0;

	/// Return the extended information on the camera.
	/** (Abvent private). */
	virtual const class CAtlCameraExObsolete*	obsoleteGetCameraEx(EExtendedVersionATL inVersion = kATLEx_Version) const = 0;
	/// Return the extended information on the camera.
	/** (Abvent private). */
	virtual class CAtlCameraExObsolete*			obsoleteGetCameraEx(EExtendedVersionATL inVersion = kATLEx_Version) = 0;

// End of API 1
	/// Put an XML representation of the camera inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef					putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the camera with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool						initWithXML(XMLNodeRef inFatherNode) = 0;

	/// Return the flags of the view
	virtual int							getFlags() const = 0;
	/// Set the flags of the view
	virtual void						setFlags(int flags) = 0;

	/// Return the view info of this view.
	/** For consulting */
	virtual const class IAtlViewInfo*	getViewInfo() const = 0;
	/// Return the view info of this view.
	/** For setting */
	virtual class IAtlViewInfo*			getViewInfo() = 0;

	/// Return the options of the view.
	/** For consulting */
	virtual const class IAtlViewOpt*	getViewOpt() const = 0;
	/// Return the options of the view.
	/** For setting */
	virtual class IAtlViewOpt*			getViewOpt() = 0;

	/// Return the insertion object.
	/** For consulting */
	virtual const class IAtlInsertionObject*	getInsertionObject() const = 0;
	/// Return the insertion object.
	/** For setting */
	virtual class IAtlInsertionObject*			getInsertionObject() = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual							~CAtlCamera() {}
};


} // namespace  AvAtl

#endif
