/** \file
 * Header for the CAtlHeliodon interface
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 30/05/2002
 */

#ifndef CAtlHeliodon_h
#define CAtlHeliodon_h


#ifndef CAtlTime_h
	#include "CAtlTime.h"
#endif
#ifndef CAtlPlace_h
	#include "CAtlPlace.h"
#endif
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef IAtlLensFlare_h
	#include "IAtlLensFlare.h"
#endif
#ifndef IAtlSkylight_h
	#include "IAtlSkylight.h"
#endif
#ifndef CAtlColor_h
	#include "CAtlColor.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


/// Heliodon (sun study)
/**
 * \interface CAtlHeliodon
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlHeliodon
{
public:
	enum { kNbColors = 5 };

	/// Get the heliodon light state.
	virtual bool					getActiveFlag() const = 0;
	/// Set the heliodon light state.
	virtual void					setActiveFlag(bool inActive) = 0;

	/// Get current time (for getting time value).
	virtual const CAtlTime&			getCurrentTime() const = 0;
	/// Get current time (for setting time values).
	virtual CAtlTime&				getCurrentTime() = 0;

	/// Get current place (for getting place values).
	virtual const CAtlPlace&		getPlace() const = 0;
	/// Get current place (for setting place values).
	virtual CAtlPlace&				getPlace() = 0;

	/// Get the power of the heliodon sky.
	virtual double					getSunshine() const = 0;
	/// Set the power of the heliodon sky.
	/** \param inSunshine value must be between 0.0 and 1.0.
	  * 0.0 means cloudy while with 1.0 you'll have a blue sky. */
	virtual void					setSunshine(double inSunshine) = 0;

	/// Get the north angle.
	virtual double					getNorthAngle() const = 0;
	/// Set the north angle.
	/** \param inAngle the north angle [0.0..360.0]. */
	virtual void					setNorthAngle(double inAngle) = 0;

	/// Get heliodon shadow state.
	virtual bool					getShadowFlag() const = 0;
	/// Set heliodon shadow state.
	virtual void					setShadowFlag(bool inFlag) = 0;

	/// Get the darkness of the shadow.
	virtual double					getShadowPower() const = 0;
	/// Set the darkness of the shadow.
	/** \param inPower the darkness (0.0 --> light and 1.0 --> dark) */
	virtual void					setShadowPower(double inPower) = 0;

	/// Get the blur level of the shadow.
	virtual int					getShadowBlur() const = 0;
	/// Set the blur level of the shadow.
	/** Value must be between 0 and 13.*/
	virtual void					setShadowBlur(int inBlur) = 0;

	/// Control the light to do global illumination.
	virtual bool					getGlobalFlag() const = 0;
	/// Control the light to do global illumination.
	virtual void					setGlobalFlag(bool inFlag) = 0;

	/// Control the power of global illumination.
	virtual double					getGlobalPower() const = 0;
   /// Control the power of global illumination.
	virtual void					setGlobalPower(double inPower) = 0;

	/// Set the global illumination quality.
	virtual int						getGlobalQuality() const = 0;
	/// Set the global illumination quality.
	virtual void					setGlobalQuality(int inQuality) = 0;

// End of API 1

	/// Put an XML representation of the heliodon inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;

	/// Initialize the heliodon with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(XMLNodeRef inFatherNode) = 0;

	/// Get the name of the heliodon.
	/** This name must be unique. */
	virtual tUtf8					getName() const = 0;
	/// Set the name of the heliodon.
	/** This name must be unique. */
	virtual void					setName(tUtf8 inName) = 0;

	/// Get the lens flare of this heliodon.
	/** For consultation */
	virtual const IAtlLensFlare*	getLensFlare() const = 0;
	/// Get the lens flare of this heliodon.
	/** For setting */
	virtual IAtlLensFlare*			getLensFlare() = 0;

	/// Control the colored sun flag.
	virtual bool					getColoredSunFlag() const = 0;
	/// Control the colored sun flag.
	virtual void					setColoredSunFlag(bool inFlag) = 0;

	/// Get the color of the sun.
	/** inNoColor must be between 0 and 4 */
	virtual const CAtlColor&		getColor(unsigned int inNoColor) const = 0;
	/// Set the color of the sun.
	/** inNoColor must be between 0 and 4 */
	virtual void					setColor(unsigned int inNoColor, const CAtlColor& inColor) = 0;

	/// Get the radiosity contribution of this light
	virtual bool					getRadiosityContribution() const = 0;
	/// Set the radiosity contribution of this light
    virtual void					setRadiosityContribution(bool radiosity) = 0;

	/// Get the sky lignt of this heliodon.
	/** For consultation */
	virtual const IAtlSkylight*		getSkylight() const = 0;
	/// Get the sky light of this heliodon.
	/** For setting */
	virtual IAtlSkylight*			getSkylight() = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class.*/
	virtual							~CAtlHeliodon() {}
};


} // namespace  AvAtl

#endif
