/** \file
 * Header for the CAtlLight interface
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */

#ifndef CAtlLight_h
#define CAtlLight_h


#ifndef CAtlColor_h
	#include "CAtlColor.h"
#endif
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif
#ifndef IAtlLightsGroup_h
	#include "IAtlLightsGroup.h"
#endif
#ifndef IAtlLensFlare_h
	#include "IAtlLensFlare.h"
#endif

// We use this namespace for our library.
namespace  AvAtl {


// We make our enums int, so you don't have to change your project settings.
#ifdef __MWERKS__
#pragma enumsalwaysint on
#endif


/// Description of light sources.
/** \interface CAtlLight 
 * Interface to describe light information. These lights are used for calculating 
 * scene illumination.
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlLight
{
public:
	/// Light Type
	enum ELightType
	{
		ePointLight,
		eSpotLight,
		eParallelLight
	};
	/// Shadow Type
	enum EShadowType
	{
		eNoShadow,
		eTracedShadow,
		eSoftShadow
	};

	/// Get the name of the light.
	/** Do not delete this string. */
	virtual tUtf8					getName() const = 0;

	/// Set the name of the light.
	virtual void					setName(tUtf8 inName) = 0;

	/// Get the type.
	virtual ELightType				getType() const = 0;
   /// Set the type.
	virtual void					setType(ELightType) = 0;

	/// Get the light on or off state.
	virtual bool					getLightFlag() const = 0;
   /// Set the light on or off.
	virtual void					setLightFlag(bool flag) = 0;

	/// Get the color of the light.
	virtual const CAtlColor&		getColor() const = 0;
	/// Set the color of the light.
	virtual void					setColor(const CAtlColor& color) = 0;

	/// Get the power of the light.
	/** Value must be between 5000.0 (high power) and 0.0 (low power)
	    Value depends on the scale of the scene. */
	virtual double					getPower() const = 0;
	/// Set the power of the light.
	/** Value must be between 5000.0 (high power) and 0.0 (low power)
	    Value depends on the scale of the scene. */
	virtual void					setPower(double power) = 0;

	/// Get the maximum distance of illumination.
	virtual double					getNearDistance() const = 0;
   /// Set the maximum distance of illumination.
	virtual void					setNearDistance(double distance) = 0;

	/// Get the illumination limit.
	virtual double					getFarDistance() const = 0;
   /// Set the illumination limit.
	virtual void					setFarDistance(double distance) = 0;

	/// Get the light location in the 3D space.
	virtual const CAtlVector&		getPosition() const = 0;
   /// Set the light location in the 3D space.
	virtual void					setPosition(const CAtlVector& position) = 0;

	/// Get the direction of the light.
	/** Valid only for spot and parallel lights. */
	virtual const CAtlVector&		getTarget() const = 0;
	/// Set the direction of the light.
	/** Valid only for spot and parallel lights. */
	virtual void					setTarget(const CAtlVector& target) = 0;

	/// Get the cone of the spotlight.
	/** Value must be between 0.0 and 360.0. */
	virtual double					getConeAngle() const = 0;
	/// Set the cone of the spotlight.
	/** Value must be between 0.0 and 360.0. */
	virtual void					setConeAngle(double angle) = 0;

	/// Get the sharpness of the edge of the cone of the spot light.
	/** 0.0 means very diffuse while 1.0 means very sharp.
	    Value must be between 0.0 and 1.0.                */
	virtual double					getSharpness() const = 0;
	/// Set the sharpness of the edges of the cone of the spot light.
	/** 0.0 means very diffuse while 1.0 means very sharp.
	    Value must be between 0.0 and 1.0.                */
	virtual void					setSharpness(double sharpness) = 0;

	/// Get the shadow type.
	/** Soft shadow is valid only for spot lights. */
	virtual EShadowType				getShadowType() const = 0;
	/// Set the shadow type.
	/** Soft shadow is valid only for spot lights. */
	virtual void					setShadowType(EShadowType) = 0;

	/// Get the darkness of the shadow.
	/** Value must be between 0.0 (light) and 1.0 (dark).*/
	virtual double					getShadowPower() const = 0;
	/// Set the darkness of the shadow.
	/** Value must be between 0.0 (light) and 1.0 (dark).*/
	virtual void					setShadowPower(double power) = 0;

	/// Get the size of shadow map.
	/** Value must be between 0 and 13. */
	virtual int						getMapSize() const = 0;
	/// Set the size of shadow map.
	/** Value must be between 0 and 13. */
	virtual void					setMapSize(int size) = 0;

	/// Get the bias of the soft shadow.
	/** Value must be between 0 and 1.0. */
	virtual double					getBias() const = 0;
	/// Set the bias of the soft shadow.
	/** Value must be between 0 and 1.0. */
	virtual void					setBias(double bias) = 0;

	/// Control the light to perform global illumination.
	virtual bool					getGlobalFlag() const = 0;
	/// Control the light to perform global illumination.
	virtual void					setGlobalFlag(bool flag) = 0;

	/// Control the power of global illumination.
	/** Value must be between 0.01 and 1.0. */
	virtual double					getGlobalPower() const = 0;
	/// Control the power of global illumination.
	/** Value must be between 0.01 and 1.0. */
	virtual void					setGlobalPower(double power) = 0;

	/// Set the global illumination quality.
	virtual int						getGlobalQuality() const = 0;
	/// Set the global illumination quality.
	virtual void					setGlobalQuality(int quality) = 0;

// End of API 1

	/// Put an XML representation of the light inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;

	/// Initialize the light with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(XMLNodeRef inFatherNode) = 0;

	/// Control if the coordinates must not be changed.
	/** This is for interface purpose. */
	virtual bool					getCoordinatesLockedFlag() const = 0;
	/// Control if the coordinates must not be changed.
	/** This is for interface purpose. */
	virtual void					setCoordinatesLockedFlag(bool inLocked) = 0;

	/// Get user interfaces private value
	virtual int						getUIValue() const = 0;
	/// Set user interfaces private value
	virtual void					setUIValue(int inValue) = 0;

	/// Get the list of groups in witch this light is.
	/** For consultation */
	virtual const AtlUUIDMgr*		getGroupMgr() const = 0;

	/// Get the list of groups in witch this light is.
	/** For setting */
	virtual AtlUUIDMgr*				getGroupMgr() = 0;

	/// Get the lens flare of this light.
	/** For consultation */
	virtual const IAtlLensFlare*	getLensFlare() const = 0;
	/// Get the lens flare of this light.
	/** For setting */
	virtual IAtlLensFlare*			getLensFlare() = 0;

	/// Get the radiosity contribution of this light
	virtual bool					getRadiosityContribution() const = 0;
	/// Set the radiosity contribution of this light
    virtual void					setRadiosityContribution(bool radiosity) = 0;

	/// Get the uuid of this light
	virtual tUUID					getId() const = 0;
	/// Set the uuid of this light
	virtual void					setId(tUUID inId) = 0;
protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual							~CAtlLight() {}
};


#ifdef __MWERKS__
#pragma enumsalwaysint reset
#endif


} // namespace  AvAtl

#endif
