/** \file
 * Header for the interface CAtlMgr.
 * This is the main header of the ATL library.
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */

#ifndef CAtlMgr_h
#define CAtlMgr_h


// Atl types names for unicode strings and URL strings.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif

// All classes of ATL
#ifndef AtlAllClasses_h
	#include "AtlAllClasses.h"
#endif

// To use this library you have to inherit from CAtlStream.
// See CAtlFileStream as example.
#ifndef CAtlStream_h
	#include "CAtlStream.h"
#endif

// To get feedback from ATL library, you have to inherit from CAtlFeedback.
// See CSimpleFeedback in (atltest.cpp) as an example.
#ifndef CAtlFeedback_h
	#include "CAtlFeedback.h"
#endif

// To stock points and polygons yourself, you have to inherit from CAtlFeedback.
// See CFileStorage as an example.
#ifndef CAtlExtStorage_h
	#include "CAtlExtStorage.h"
#endif

// Template for color classes.
#ifndef CAtlColor_h
	#include "CAtlColor.h"
#endif

// Describe a 3d vector.
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif

// Polygon data
#ifndef CAtlPolygon_h
	#include "CAtlPolygon.h"
#endif

// Ambient color setup
#ifndef CAtlAmbient_h
	#include "CAtlAmbient.h"
#endif

// Fog setup
#ifndef CAtlAtmosphere_h
	#include "CAtlAtmosphere.h"
#endif

// Background setup
#ifndef CAtlBackground_h
	#include "CAtlBackground.h"
#endif

// Camera setup
#ifndef CAtlCamera_h
	#include "CAtlCamera.h"
#endif

// Property setup
#ifndef CAtlProperty_h
	#include "CAtlProperty.h"
#endif

// Light setup
#ifndef CAtlLight_h
	#include "CAtlLight.h"
#endif

// Animation setup
#ifndef IAtlCameraPath_h
	#include "IAtlCameraPath.h"
#endif

// Description of a file type
#ifndef CAtlFileType_h
	#include "CAtlFileType.h"
#endif

// Object collection template class.
#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


// We make our enums int, so you don't have to change your project settings.
#ifdef __MWERKS__
#pragma enumsalwaysint on
#endif


// Declarations of object lists.

/// List of points
typedef TAtlObjectMgr<class CAtlVector>			AtlPointMgr;

/// List of polygons
typedef TAtlObjectMgr<class CAtlPolygon>		AtlPolygonMgr;

/// List of cameras
typedef TAtlObjectMgr<class CAtlCamera>			AtlCameraMgr;

/// List of properties
typedef TAtlObjectMgr<class CAtlProperty>		AtlPropertyMgr;

/// List of lights
typedef TAtlObjectMgr<class CAtlLight>			AtlLightMgr;

/// List of animations
typedef TAtlObjectMgr<class IAtlAnimation>		AtlAnimationsMgr;

/// List of camera path
typedef TAtlObjectMgr<class IAtlCameraPath>		AtlCameraPathMgr;

/// List of vr panorama cameras
typedef TAtlObjectMgr<class IAtlPanoNode>		AtlPanoNodeMgr;

/// List of vr panorama camera links
typedef TAtlObjectMgr<class CAtlPanoLink>		AtlPanoLinkMgr;

/// List of vr objects
typedef TAtlObjectMgr<class CAtlViewObject>		AtlViewObjectMgr;

/// List of parallels views
typedef TAtlObjectMgr<class IAtlParallelView>	AtlParallelViewMgr;

/// List of names (some objects are linked by name)
typedef TAtlObjectMgr<class IAtlLightsGroup>	AtlLightsGroupMgr;

/// List of heliodon
typedef TAtlObjectMgr<class CAtlHeliodon>		AtlHeliodonMgr;

/// List of panoramas
typedef TAtlObjectMgr<class IAtlPanorama>		AtlPanoramasMgr;

/// List of objects
typedef TAtlObjectMgr<class IAtlObjectInstance>	AtlObjectInstancesMgr;

/// List of objects
typedef TAtlObjectMgr<class IAtlImage>			AtlImagesMgr;


class CAtlMatrix;


/// This class is used to process Art*lantis data.
/** \interface CAtlMgr 
 * This class describes the manager that allows you to access the different managers
 * of each object that an Art*lantis document can contain.
 * This interface includes all the ATL interfaces
 * 
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlMgr
{
public:
	/// Result value from read and write.
	enum ERWResultType
	{
		/// The io operation is successful.
		eRWSuccess,
		/// An error occured while reading.
		eRWReadError,
		/// An error occured while writing.
		eRWWriteError
	};

	/// Create an CAtlMgr.
	/** Use this function to create a CAtlMgr.
	 * \param feedback a listener for warnings generated by atl.
	 * \param extStorage a listener for private management of points and polygons.
	 * \return the created AtlMgr
	 * \retval NULL if the client version isn't supported. */
	static CAtlMgr*					newCAtlMgr(CAtlFeedback* feedback = 0, CAtlExtStorage* extStorage = 0)
												{ return newCAtlMgr(kATL_Version, feedback, extStorage); }

	/// Create an CAtlMgr specifying the version.
	/** Use this function to create a CAtlMgr and specify the API version that you support.
	 * This is an unrecommended usage. */
	static CAtlMgr*					newCAtlMgrWithVersion(EVersionATL version, CAtlFeedback* feedback, CAtlExtStorage* extStorage)
												{ return newCAtlMgr(version, feedback, extStorage); }

   /// Use these functions to delete a CAtlMgr.
	virtual void					destroy() = 0;

	/// Check the readability of the stream.
	/** Use this function to determine if this stream can be read by the library.
	 * You can use this function to determine which files can be read.
	 * \param readStream a stream to the data.
	 * \param fmtName a buffer for the name of the format.
	 * \param fmtNameSize the size of the buffer.
	 * \retval true when the file is recognized and set fmtName to the name of the format.
	 * \retval false when the format isn't recognized. */
	static AVATL_EXP bool AVATL_CALL_CONVENTION checkFormat(CAtlStream* readStream, tUtf8Char* fmtName = 0, int fmtNameSize = 0);

	/// Check the readability of the stream.
	/** Use this function to determine if this stream can be read by the library.
	 * You can use this function to determine which files can be read.
	 * \param readStream a stream to the data.
	 * \param fmtName a string that will be set to the format name.
	 * \retval true when the file is recognized and set fmtName to the name of the format.
	 * \retval false when the format isn't recognized. */
	static bool						checkFormat(CAtlStream* readStream, tUtf8String* fmtName)
									{
										tUtf8Char	s[256];
										bool	r = checkFormat(readStream, s, sizeof(s));
										*fmtName = s;
										return	r;
									}

	/// List of known file types.
	/** Use this function to get a list of recognized file types.
	 * You can use this list in your open dialog box.
	 * The first one is the normal extension when you are saving files.
	 * Do not attempt to delete the list or any item.
	 * The list ends with a NULL ptr.*/
	static AVATL_EXP const CAtlFileType* const* AVATL_CALL_CONVENTION getFileTypeList();

	/// Return the version of the ATL library.
	static AVATL_EXP EVersionATL AVATL_CALL_CONVENTION getATLVersion();

	/// Return the extended version of the ATL library.
	static AVATL_EXP EExtendedVersionATL AVATL_CALL_CONVENTION getExtendedAtlVersion();

	/// Return the preview image of this file stream.
	/** \retval NULL if there are no preview image in the file. */
	static AVATL_EXP IAtlImage* AVATL_CALL_CONVENTION readPreviewImage(CAtlStream* inReadStream);

	/// Set the information about the application that created this ATL document.
	virtual void					setApplicationInfo(tUtf8 inApplName, tUtf8 inVersionStr) = 0;

	/// Get the extended manager.
	/** Use this to get the extended version of the manager (Abvent private).
	 * \return a pointer to the extended manager.
	 * \retval NULL if the extended client version isn't supported. */
	virtual class CAtlMgrEx*		getExtendedAtlMgr(EExtendedVersionATL version = kATLEx_Version) = 0;

	/// Read from the stream.
   /** Use this function to read.
    * \param readStream the stream to read from
    * \retval eRWSuccess if reading is succesful.
    * \retval eRWReadError if reading is unsuccesful. */
	virtual ERWResultType			read(CAtlStream* readStream) = 0;
	/// Write to the stream.
   /** Use this function to write.
    * \param writeStream the stream to write to
    * \retval eRWSuccess if writing is successful.
    * \retval eRWWriteError if writing is unsuccessful. */
	virtual ERWResultType			write(CAtlStream* writeStream) = 0;  

	/// Get the read version number.
	/** Use this function to get info on the read version (Currently 3.5 or 4.0, 4.5, 4.6).
	 * \param outMajor pointer to the integer that will receive the major number.
	 * \param outMinor pointer to the integer that will receive the minor number.
	 * \retval true if we have read something
	 * \retval false if no reading done. */
	virtual bool					getReadVersion(int* outMajor, int* outMinor) const = 0;

	/// The Ambient setup. 
	virtual CAtlAmbient*			getAmbient() = 0;
	/// The atmosphere(fog) setup. 
	virtual CAtlAtmosphere*			getAtmosphere() = 0;
	/// The background setup.
	virtual CAtlBackground*			getBackground() = 0;

	/// List of Cameras
	virtual AtlCameraMgr*			getCameraMgr() = 0;
	/// List of Polygons
	virtual AtlPolygonMgr*			getPolygonMgr() = 0;
	/// List of Points
	virtual AtlPointMgr*			getPointMgr() = 0;
	/// List of Properties
	virtual AtlPropertyMgr*			getPropertyMgr() = 0;
	/// List of Lights
	virtual AtlLightMgr*			getLightMgr() = 0;

	// Data from this point aren't garanteed to be upward compatible supported.

	/// List of Animations
	virtual AtlCameraPathMgr*		obsoleteGetAnimationMgr() = 0;

	/// List of Panoramas
	virtual AtlPanoNodeMgr*			obsoleteGetPanoramaMgr() = 0;

	/// List of links between panoramas
	virtual AtlPanoLinkMgr*			obsoleteGetPanoLinkMgr() = 0;

	/// List of vr objects
	virtual AtlViewObjectMgr*		getViewObjectMgr() = 0;

	/// The heliodon setup
	/** Now return the first heliodon of the list */
	virtual class CAtlHeliodon*		obsoleteGetHeliodon() = 0;

	/// List of parallels views.
	virtual AtlParallelViewMgr*		getParallelViewMgr() = 0;

// End of API 1
	/// List of heliodons
	virtual AtlHeliodonMgr*			getHeliodonMgr() = 0;

	/// List of the names of the groups of lights.
	/** Entry 0 must be "" (shown in interface as "default" depending of the current language) */
	virtual AtlLightsGroupMgr*		getLightsGroupsMgr() = 0;

	/// Lists of animations
	virtual AtlAnimationsMgr*		getAnimationsMgr() = 0;

	/// Lists of panoramas
	virtual AtlPanoramasMgr*		getPanoramasMgr() = 0;

	/// Get the current transformation matrix.
	virtual const CAtlMatrix&		getTransformation() const = 0;

	/// Set the transformation matrix.
	virtual void					setTransformation(const CAtlMatrix& inMatrix) = 0;

	/// Return the preview image of this AtlMgr.
	virtual IAtlImage&				getPreviewImage() = 0;

	/// Return the images manager
	virtual AtlImagesMgr&			getImagesMgr() = 0;

	/// Return the definition of the AtlObject (if this document is an object).
	virtual class IAtlAtlObjectDefinition*	getAtlObjectDefinition() = 0;

	/// Return the objets used by this documents.
	virtual AtlObjectInstancesMgr*	getObjectInstancesMgr() = 0;

	/// Return the current language (2 char, standard ISO639)
	/** By default, value is "en" for english */
	virtual const char*				getLanguage() const = 0;
	/// Set the current language (2 char, standard ISO639)
	virtual void					setLanguage(const char* inLanguage) = 0;

	/// Return the comment specified by the index.
	/** Return NULL if there are no more comments */
	virtual tUtf8					getComment(unsigned int inIndex) const = 0;

	/// Change the comment specified by the index.
	/** If there are no comment for the specified index, the comment will be added at the end.
	  * Return the index of the comment affected.
	  * If the comment is NULL, the comment specified by the index will be destroyed. */
	virtual unsigned int			setComment(tUtf8 inComment, unsigned int inIndex = (unsigned int)-1) = 0;

	/// Return the URL where the file was saved
	/** This path will be used to help relocalizations */
	virtual tURL					getURL() const = 0;

	/// Set the URL where the file will be saved
	/** This path will be used to help relocalizations */
	virtual void					setURL(tURL inURL) = 0;

	/// Return the XML Element for the specified Application Data tag
	/** The node will be created if there was no node present */
	virtual XMLNodeRef				getApplicationData(tUtf8 inApplicationTag) = 0;

private:
	/** Then the library will know which api your application will support. */
	static AVATL_EXP CAtlMgr* AVATL_CALL_CONVENTION newCAtlMgr(EVersionATL version, CAtlFeedback* feedback, CAtlExtStorage* extStorage);

protected:
	/** Illegal to delete a CAtlMgr. You must use destroy(). */
	virtual							~CAtlMgr() {}
};

#ifdef __MWERKS__
#pragma enumsalwaysint reset
#endif


} // namespace  AvAtl


/* C API: This api has been created on Graphisoft request. We recommend to use directly the CAtlMgr. */
extern "C"
{
/// C function equivalent to CAtlMgr::newCAtlMgr
extern AVATL_EXP AvAtl::CAtlMgr* AVATL_CALL_CONVENTION
    atl_new_mgr(AvAtl::EVersionATL version, AvAtl::CAtlFeedback* feedback, AvAtl::CAtlExtStorage* extStorage);
/// C function equivalent to CAtlMgr::checkFormat
extern AVATL_EXP bool AVATL_CALL_CONVENTION
    atl_check_format(AvAtl::CAtlStream* readStream, AvAtl::tUtf8Char* fmtName, int fmtNameSize);
/// C function equivalent to CAtlMgr::getFileTypeList
extern AVATL_EXP const AvAtl::CAtlFileType* const* AVATL_CALL_CONVENTION
    atl_get_file_type_list();
/// C function equivalent to CAtlMgr::getATLVersion
extern AVATL_EXP AvAtl::EVersionATL AVATL_CALL_CONVENTION
    atl_get_version();
/// C function equivalent to CAtlMgr::getExtendedAtlVersion
extern AVATL_EXP AvAtl::EExtendedVersionATL AVATL_CALL_CONVENTION
    atl_get_extended_version();
/// C function equivalent to CAtlMgr::readPreviewImage
extern AVATL_EXP AvAtl::IAtlImage* AVATL_CALL_CONVENTION
    atl_read_preview_image(AvAtl::CAtlStream* inReadStream);
}


#endif
