/** \file
 * Header for the CAtlProperty interface
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */

#ifndef CAtlProperty_h
#define CAtlProperty_h


// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef CAtlColor_h
	#include "CAtlColor.h"
#endif
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


// We make our enums int, so you don't have to change your project settings.
#ifdef __MWERKS__
#pragma enumsalwaysint on
#endif


/// Description of a property than can be applied to polygons.
/** \interface CAtlProperty 
 *	The information on a property
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlProperty
{
public:
   /// Mapping type.
	enum EMappingType
	{
		eNoTexturePresent = -1,
		eOrthogonalCubic,
		eOrthogonalPlanar,
		eVertical,
		eSpherical,
		eXCylindar,
		eYCylindar,
		ePlanetar,
		eUVMapping
	};

	/// Return the name of the texture.
	/** Do not delete this string. */
	virtual tUtf8					getName() const = 0;

	/// Set the name of the texture.
	virtual void					setName(tUtf8 inName) = 0;

	/// Control the visibility of the objects of that property.
	virtual bool					getVisibility() const = 0;
	/// Control the visibility of the objects of that property.
	virtual void					setVisibility(bool inFlag) = 0;

	/// Control the shadow produced by the objects of that property.
	virtual bool					getShadows() const = 0;
	/// Control the shadow produced by the objects of that property.
	virtual void					setShadows(bool inFlag) = 0;

	/// Control the smooth aspect of objects of that property.
	/** The value must be between 0.01 and 1.0.
	    1.0 means no smoothing, 0.01 means full smoothing. */
	virtual double					getSmooth() const = 0;
	/// Control the smooth aspect of the objects of that property.
	/** The value must be between 0.01 and 1.0.
	    1.0 means no smoothing, 0.01 means full smoothing. */
	virtual void					setSmooth(double inSmooth = 0.5) = 0;

	/// Simple scale factor. This value must be positive.
	virtual double					getScale() const = 0;
	/// Simple scale factor. This value must be positive.
	virtual void					setScale(double inScale) = 0;

	/// Control the first color.
	virtual CAtlColor				getDiffuseColor() const = 0;
	/// Control the first color.
	virtual void					setDiffuseColor(const CAtlColor& inColor) = 0;

	/// Control the specular color.
	virtual CAtlColor				getSpecularColor() const = 0;
	/// Control the specular color.
	virtual void					setSpecularColor(const CAtlColor& inSpecular) = 0;

	/// Control the specular power.
	/** power value must be positive and it is recommended to be inferior to 100.0.
		Please use the other getGlossy() */
	virtual void					getGlossyObsolete(double* outPower) const = 0;
	/// Control the specular power.
	/** power value must be positive and it is recommended to be inferior to 100.0. */
	virtual void					setGlossy(double inPower) = 0;

	/// Control the transparency.
	/** Each component can have a different transparency factor.
	    Value for each component must be between 0 (opaque) and 255 (transparent).*/
	virtual CAtlColor				getTransparentColor() const = 0;
	/// Control the transparency.
	/** Each component can have a different transparency factor.
	    Value for each component must be between 0 (opaque) and 255 (transparent).*/
	virtual void					setTransparentColor(const CAtlColor& inColor) = 0;

	/// Control the reflection.
	/** Each component can have a different reflection factor.
	    Value for each component must be between 0 (no reflection) and 255 (full reflection).*/
	virtual CAtlColor				getReflectionColor() const = 0;  
	/// Control the reflection.
	/** Each component can have a different reflection factor.
	    Value for each component must be between 0 (no reflection) and 255 (full reflection).*/
	virtual void					setReflectionColor(const CAtlColor& inColor) = 0;

	/// Control the use of the texture. Return NULL if no texture set.
	virtual tURL					getTextureURL() const = 0;
	/// Control the use of the texture.
	virtual void					setTextureURL(tURL inURL) = 0;

	/// Control the mapping methods of the texture.
	/** Return eNoTexturePresent if no texture set. */
	virtual EMappingType			getMapping() const = 0;
	/// Control the mapping methods of the texture.
	virtual void					setMapping(EMappingType inMapping = eOrthogonalCubic) = 0;

	/// Control the placement of the texture. Return false if no texture set.
	virtual bool					getTexturePlacement(CAtlVector* inOrigin, CAtlVector* inXAxis, CAtlVector* inYAxis) const = 0;
	/// Control the placement of the texture. Return false if no texture set.
	virtual void					setTexturePlacement(const CAtlVector& inOrigin, const CAtlVector& inXAxis, const CAtlVector& inYAxis) = 0;

	/// Control the color to be the transparent color of the texture.
	/** Return false if no texture set. */
	virtual bool					getTextureTransparentColor(CAtlColor* outColor) const = 0;
	/// Control the color to be the transparent color of the texture.
	virtual void					setTextureTransparentColor(const CAtlColor& inColor) = 0;

	/// Control the repetition of the texture.
	/** Return false if no texture set. */
	virtual bool					getTextureRepetition(bool* outRepeatX, bool* outRepeatY) const = 0;
	/// Control the repetition of the texture.
	virtual void					setTextureRepetition(bool inRepeatX, bool inRepeatY) = 0;

	/// Control the invertion of the texture.
	/** Return false if no texture set. */
	virtual bool					getTextureInvertion(bool* outXInverted, bool* outYInverted) const = 0;
	/// Control the invertion of the texture.
	virtual void					setTextureInvertion(bool inXInverted = false, bool inYInverted = false) = 0;

	/// Get the scale of the texture.
	/** Return false if no texture set. */
	virtual bool					getTextureScale(double* outWidth, double* outHeight) const = 0;
	/// Set the scale of the texture.
	virtual void					setTextureScale(double inWidth, double inHeight) = 0;

	/// Return the TCE object.
	/** For usage by Abvent only */
	virtual const class CAtlTCE*	getTCE() const = 0;
	/// Return the TCE object.
	/** For usage by Abvent only */
	virtual class CAtlTCE*			getTCE() = 0;

	/// Get the angle of the texture.
	/** Return false if no texture set. */
	virtual bool					getTextureAngle(double* outAngle) const = 0;
	/// Set the angle of the texture.
	virtual void					setTextureAngle(double inAngle) = 0;

	/// Copy the property inOther in this property.
	/** Return false if the copy failed */
	virtual bool					loadFrom(const CAtlProperty& inOther) = 0;

	/// Control the flag that this property is for billboard.
	virtual bool					getIsBBProperty() const = 0;

	/// Control the flag that this property is for billboard.
	virtual void					setIsBBProperty(bool inIsBBProperty) = 0;

	/// Control the glossy power.
	/** power value must be positive and it is recommended to be inferior to 100.0. */
	virtual double					getGlossy() const = 0;

	/// Control the neon color.
	virtual CAtlColor				getNeonColor() const = 0;
	/// Control the neon color.
	virtual void					setNeonColor(const CAtlColor& inNeonColor) = 0;

	/// Control the neon.
	virtual double					getNeon() const = 0;
	/// Control the neon.
	virtual void					setNeon(double inNeon) = 0;

	/// Control the refraction.
	virtual double					getRefraction() const = 0;
	/// Control the refraction.
	virtual void					setRefraction(double inRefraction) = 0;

// End of API 1

	/// Put an XML representation of the property inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;

	/// Initialize the property with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(XMLNodeRef inFatherNode) = 0;

	/// Get the id of the property.
	/** Used with embeded Atl Object
		For usage by Abvent only 
		This id is the universal unique id of the property. */
	virtual tUUID					getId() const = 0;
	/// Set the id of the property.
	/** Used with embeded Atl Object
		For usage by Abvent only
		This id is the universal unique id of the property. */
	virtual void					setId(tUUID inId) = 0;

	/// Get the id of the overloaded property.
	/** Used with embeded Atl Object
		For usage by Abvent only
		This id is an id used as an index in the property manager. For examples, with 3d plants objects,
		when the season changes, we have to change properties. This id indicates where we have to put the new property in the property manager. 
		See below for more explications */
	virtual tUUID					getOverloadedPropId() const = 0;
	/// Set the id of the overloaded property.
	/** Used with embeded Atl Object
		For usage by Abvent only */
	virtual void					setOverloadedPropId(tUUID inId) = 0;

	/// Get the overload id of this overload.
	/** Used with embeded Atl Object
		For usage by Abvent only 
		This id is the "collection's" id : for examples, with 3d plants objects, there is an uuid corresponding to each seasons :
		All properties which are needed for the season "summer" will have the same overload id. 
		See below for more explications */
	virtual tUUID					getOverloadId() const = 0;
	/// Get the overload id of this overload.
	/** Used with embeded Atl Object
		For usage by Abvent only */
	virtual void					setOverloadId(tUUID inId) = 0;

	/// Control that objects of that property received shadows
	virtual bool					getReceiveShadows() const = 0;
	/// Control that objects of that property received shadows
	virtual void					setReceiveShadows(bool inFlag) = 0;

	/// Indicate that UV is defined for all triangles.
	virtual bool					getCanMapWithUV() const = 0;
	/// Indicate that UV is defined for all triangles.
	virtual void					setCanMapWithUV(bool inFlag) = 0;

	/// Control the angular LOD shift.
	virtual bool					getTextureAngularLODShift(double* outALS) const = 0;
	/// Control the angular LOD shift.
	virtual void					setTextureAngularLODShift(double inALS) = 0;

	/// Control the linear LOD shift.
	virtual bool					getTextureLinearLODShift(double* outLLS) const = 0;
	/// Control the linear LOD shift.
	virtual void					setTextureLinearLODShift(double inLLS) = 0;

	/// Indicate that alpha is used for transparency
	virtual bool					getUseAlphaForTransparency() const = 0;
	/// Set the flag that alpha is used for transparency
	virtual void					setUseAlphaForTransparency(bool inUse) = 0;

	/// Return the reflection limit
	virtual double					getReflectionLimit() const = 0;
	/// Set the reflection limit
	virtual void					setReflectionLimit(double inLimit) = 0;

	/// Return the light emitting color
	virtual const CAtlColor			getEmittingColor() const = 0;
	/// Set the light emitting color
	virtual void					setEmittingColor(const CAtlColor& inColor) = 0;

	/// Return the light emitting power
	virtual double					getEmittingPower() const = 0;
	/// Set the light emitting power
	virtual void					setEmittingPower(double inPower) = 0;

	/// Set this property to a "display" mode
	virtual void					setTextureDisplayMode(bool inFlag) = 0;
	/// Control that the property is in a display mode
	virtual bool					getTextureDisplayMode() const = 0;


protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual							~CAtlProperty() {}
};


/**
	Explications of the uuids concepts in CAtlProperty.
	Properties use 3 uuids :
	_ The id.
	_ The Overload Prop Id.
	_ The Overload Id.

	The id is the real id of the property. It is unique and universal. When you create a property, a new id is created for it.
	The overload prop id is an id used as an index in the property manager. Like the id, a new overload prop id is set when the property is created.
	The overload index is a collection's id. You must set this id, after creating the property, if you want to do a collection.

	For example, we will take a 3d plant object which can have 2 seasons : Summer and Winter, and 2 properties.
	By default, this plant is in winter.
	The plant have 2 property manager : the current property manager, and the property manager of the plant's definition.

	The structure of the current property manager in the plant is like : (note that uuids are noted like string, but it is just for reading easily.
	_ Property 0 : Id "1", Overload Prop Id "A", OverloadId "Winter".
	_ Property 1 : Id "2", Overload Prop Id "B", OverloadId "Winter".

	Now, in the property manager of the plant's definition, we have 4 properties :
	_ Property 0 : Id "1", Overload Prop Id "A", OverloadId "Winter".
	_ Property 1 : Id "2", Overload Prop Id "B", OverloadId "Winter".
	_ Property 2 : Id "3", Overload Prop Id "A", OverloadId "Summer".
	_ Property 3 : Id "4", Overload Prop Id "B", OverloadId "Summer".

	For a plant, ther is a "plant setup manager" wich indicates the overload id corresponding to a season.
	So, when we changes the season of the plant, we do :
	_ Get the overload id of the season, in the plant setup manager.
	_ For each property "P1" in the property manager of the plant's definition we have :
	   _ if the overload id of the property "P1" is the good overload id
	       _ For each property "P2" in the current property manager of the plant
		      _ If the overload prop id of the property "P2" is the same that the overload prop id of the property "P1"
			     _ We replace "P2" by "P1".
    So for our plant, if we want to have the plant in Summer, the structure of the current Property manager will be :
	_ Property 0 : Id "3", Overload Prop Id "A", OverloadId "Summer".
	_ Property 1 : Id "4", Overload Prop Id "B", OverloadId "Summer".
*/
#ifdef __MWERKS__
#pragma enumsalwaysint reset
#endif


} // namespace  AvAtl

#endif
