/** \file
 * Header for the CAtlStream template
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Cong Loc Kieu
 * \version 1.0
 * \date 04/02/2002
 */

#ifndef CAtlStream_h
#define CAtlStream_h

// We use this namespace for our library.
namespace  AvAtl {

#ifdef __MWERKS__
#pragma enumsalwaysint on
#endif


/// ATL stream Interface to read and write ATL data.
/** \interface CAtlStream 
 * You must inherit from the interface CAtlStream to use the ATL library.
 * This allows you to perform any IO you want.
 * It will be easy for you to write your stream because we request very
 * simple IO operations. Stream must be binary for future compatibility.
 * For an example of use, see the CAtlFileStream class.
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Cong Loc Kieu
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlStream
{
public:
	/// Type of requested stream
	enum ERWMode
	{
		eRead = 0,
		eWrite
	};

	/// Version of this header.
	enum { eCurrentStreamVersion = 1 };

   /// This function will be called to ask for the shortname of the current stream.
   virtual bool  getShortName(char * shortName, int buffSize) throw() = 0;

   /// Will be called to create a new stream relative to the original stream.
   /** In writing mode, you must create and/or truncate. */
   virtual CAtlStream* clone(const char *newShortName, ERWMode mode) throw() = 0;

   /// Destroy will be called to delete the cloned stream.
   /** This function will not be called for the original stream
       (the one passed to the read and write method in the CAtlMgr class). */
   virtual void   destroy() throw() = 0;

   /// Will be called to set the Mac file type.
   /** Called only for writing the stream. */
   virtual void   setTypeAndCreator(long type, long creator) throw() = 0;

	/// Read one byte (8 bits) in the stream (increment the stream pointer accordingly).
	/** Return -1 if the stream is at the end. */
	int				read()
							{
								char	b;
								if (read(&b, 1) != 1)
									return -1;
								return (unsigned char)b;
							}

   /// Read nbBytes bytes (8 bits) in the stream (increment the stream pointer accordingly).
	/** Return the number of bytes read.*/
	virtual int		read(char* buffer, int nbBytes) throw() = 0;

	/// Write one byte (8 bits) in the stream (increment the stream pointer accordingly).
	void				write(int byte)  throw()
							{
								char	b = (char)byte;
								write(&b, 1);
							}

	/// Write nbBytes bytes (8 bits) in the stream (increment the stream pointer accordingly).
	virtual void	write(const char* buffer, int nbBytes) throw() = 0;

	/// Get the stream pointer (offset from the beginning).
	/** Return -1 if an error occurs. */
	virtual long	getPosition() throw() = 0;

	/// Move the stream pointer to the position specified (from the beginning).
	virtual void	setPosition(long pos) throw() = 0;

	/// Return the version of the stream
	virtual int		getSteamVersion() { return eCurrentStreamVersion; }

protected:
	/** Atl will delete cloned streams via the destroy() function. */
	virtual			~CAtlStream() {};
};

#ifdef __MWERKS__
#pragma enumsalwaysint reset
#endif

} //end namespace

#endif
