/** \file
 * Header for the CAtlViewObject interface
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 30/05/2002
 */

#ifndef CAtlViewObject_h
#define CAtlViewObject_h


#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


/// Information on view object (QuickTime VR).
/** \interface CAtlViewObject 
 *	The information on view object (QuickTime VR).
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 30/05/2002
 */
class CAtlViewObject
{
public:
	/// Return the name of the VR Object.
	/** Do not delete this string. */
	virtual tUtf8				getName() const = 0;

	/// Set the name of the vue object.
	virtual void				setName(tUtf8 inName) = 0;

	/// Get the view flag.
	virtual int					getFlags() const = 0;
	/// Set the view flag
	virtual void				setFlags(int inFlag) = 0;

	/// Get the focal angle of the camera.
	virtual double				getFocal() const = 0;
	/// Set the focal angle of the camera.
	virtual void				setFocal(double inFocal) = 0;

	/// Get the target location in 3D space.
	virtual const CAtlVector&	getCenter() const = 0;
	/// Set the target location in 3D space.
	virtual void				setCenter(const CAtlVector& inCenter) = 0;

	/// Get the radius between the cameras and the target.
	virtual double				getRadius() const = 0;
	/// Set the radius between the cameras and the target.
	virtual void				setRadius(double inRadius) = 0;

	/// Get the horizontal angle.
	/** Value must be between 0 and 360 */
	virtual int					getHorizontalAngle() const = 0;
	/// Set the horizontal angle.
	/** Value must be between 0 and 360 */
	virtual void				setHorizontalAngle(int inAngle) = 0;

	/// Get the vertical angle.
	/** Value must be between 0 and 360 */
	virtual int					getVerticalAngle() const = 0;
	/// Set the vertical angle.
	/** Value must be between 0 and 360 */
	virtual void				setVerticalAngle(int inAngle) = 0;

	/// Get the horizontal angle step.
	/** Value must be between 0 and 360 */
	virtual int					getHorizontalStep() const = 0;
	/// Set the horizontal angle step.
	/** Value must be between 0 and 360 */
	virtual void				setHorizontalStep(int inStep) = 0;

	/// Get the vertical angle step.
	/** Value must be between 0 and 360 */
	virtual int					getVerticalStep() const = 0;
	/// Set the vertical angle step.
	/** Value must be between 0 and 360 */
	virtual void				setVerticalStep(int inStep) = 0;

	/// Get the displacement on horizontal.
	/** Value must be between 0 and 360 */
	virtual int					getHorizontalShift() const = 0;
	/// Set the displacement on horizontal.
	/** Value must be between 0 and 360 */
	virtual void				setHorizontalShift(int inDisplacementAngle) = 0;

	/// Get the displacement on vertical.
	/** Value must be between 0 and 360 */
	virtual int					getVerticalShift() const = 0;
	/// Set the displacement on vertical.
	/** Value must be between 0 and 360 */
	virtual void				setVerticalShift(int inDisplacementAngle) = 0;

	/// Get the horizontal start angle.
	virtual double				getHorizontalStart() const = 0;
	/// Set the horizontal start angle.
	virtual void				setHorizontalStart(double inStartAngle) = 0;

	/// Get the vertical start angle.
	virtual double				getVerticalStart() const = 0;
	/// Set the vertical start angle.
	virtual void				setVerticalStart(double inStartAngle) = 0;

	/// Get the start angle on the longitude.
	virtual int					getOriginAngle() const = 0;
	/// Set the start angle on the longitude.
	virtual void				setOriginAngle(int inOriginAngle) = 0;

// End of API 1

	/// Put an XML representation of the view object inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef			putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the view object with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool				initWithXML(XMLNodeRef inFatherNode) = 0;

	/// Return the view info of this view.
	/** For consulting */
	virtual const class IAtlViewInfo*	getViewInfo() const = 0;
	/// Return the view info of this view.
	/** For setting */
	virtual class IAtlViewInfo*			getViewInfo() = 0;

	/// Return the options of the view.
	/** For consulting */
	virtual const class IAtlViewOpt*	getViewOpt() const = 0;
	/// Return the options of the view.
	/** For setting */
	virtual class IAtlViewOpt*			getViewOpt() = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class.*/
	virtual						~CAtlViewObject() {}
};


} // namespace  AvAtl

#endif
