/** \file
 * Header for the IAtlCameraPath interface
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 30/05/2002
 */

#ifndef IAtlCameraPath_h
#define IAtlCameraPath_h


#ifndef CAtlCamKey_h
	#include "CAtlCamKey.h"
#endif
#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


/// List of points
typedef TAtlObjectMgr<class CAtlVector>		AtlPointMgr;

/// Animation done by camera displacement.
/** \interface IAtlCameraPath 
 *	The information on a animation
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 30/05/2002
 */
class IAtlCameraPath
{
public:
	/// List of camera key
	typedef TAtlObjectMgr<class CAtlCamKey>		AtlCamKeyMgr;

	/** Return the name of the animation.
		 Don't delete this string. */
	virtual tUtf8				getName() const = 0;

	/// Set the name of the animation.
	virtual void				setName(tUtf8 inName) = 0;

	/// Is the main sequence ?
	virtual bool				getSequenceMainFlag() const = 0;
	/// Set the sequence as main (if true) ?
	/** Only one sequence is the main. */
	virtual void				setSequenceMainFlag(bool inFlag) = 0;

	/// Is the sequence is active ?
	virtual bool				getSequenceOnFlag() const = 0;
	/// Set the sequence active state.
	virtual void				setSequenceOnFlag(bool) = 0;

	/// Get the duration of the movie.
	/** Return the duration is in seconds. */
	virtual double				getDuration() const = 0;
	/// Set the duration of the movie.
	/** Value is in seconds and must be positive. */
	virtual void				setDuration(double inDuration) = 0;

	/// Get the number of extra images that come after the duration in seconds.
	virtual int					getNbXImages() const = 0;
	/// Set the number of extra images that come after the duration in seconds.
	virtual void				setNbXImages(int inNbXimages) = 0;

	/// Get progressive speed state.
	/** return true if speed is progressive. */
	virtual bool				getSpeedFlag() const = 0;
	/// Set the progressive speed state.
	/** inSpeedFlag true -> speed must be progressive. */
	virtual void				setSpeedFlag(bool inSpeedFlag) = 0;

	/// Get the speed.
	/** Value is in km/h. */
	virtual double				getSpeed() const = 0;
	/// Set the speed.
	/** inSpeed must be in km/h. */
	virtual void				setSpeed(double inSpeed) = 0;

	/// Get the closing path state.
	/// If true the animation path will be closed.
	virtual bool				getClosingFlag() const = 0;
	/// Set the closing path state.
	/// If true the animation path will be closed.
	virtual void				setClosingFlag(bool cflag) = 0;

	/// Get the list of point of the path.
	/** The path is described by a 3D Bezier curve. */
	virtual AtlPointMgr*		getPointList() = 0;
	/// Get the list of point of the path.
	/** The path is described by a 3D Bezier curve. */
	virtual const AtlPointMgr*	getPointList() const = 0;

	/// Get the list of camera state.
	virtual AtlCamKeyMgr*		getCamKeyList() = 0;
	/// Get the list of camera state.
	virtual const AtlCamKeyMgr*	getCamKeyList() const = 0;

// End of API 1

	/// Put an XML representation of the animation inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef			putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the animation with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool				initWithXML(XMLNodeRef inFatherNode) = 0;

	/// Return the view info of this view.
	/** For consulting */
	virtual const class IAtlViewInfo*	getViewInfo() const = 0;
	/// Return the view info of this view.
	/** For setting */
	virtual class IAtlViewInfo*			getViewInfo() = 0;

	/// Return the current time of this animation.
	virtual double				getCurrentTime() const = 0;
	/// Set the current time for this animation.
	virtual void				setCurrentTime(double inTime) = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class.*/
	virtual						~IAtlCameraPath() {}
};


} // namespace  AvAtl

#endif
