/** \file
 * Header for the IAtlObjectInstance interface
 *
 * \author Abvent R&D Copyright (c) 2004
 * \author Richard Young
 * \version 1.0
 * \date 26/04/2004
 */

#ifndef IAtlObjectInstance_h
#define IAtlObjectInstance_h


// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif
#ifndef	CAtlTriangle_h
	#include "CAtlTriangle.h"
#endif
#ifndef	IAtlAtlObjectDefinition_h
	#include "IAtlAtlObjectDefinition.h"
#endif
#ifndef	CAtlBoundingBox_h
	#include "CAtlBoundingBox.h"
#endif

// We use this namespace for our library.
namespace  AvAtl {


// Forward declaration
class IAtlAtlObjectInstance;
class IAtl3DPlantRef;
class IAtlBBObjectInstance;
class IAtlObjectLightRef;


/// Manager of instances
typedef TAtlObjectMgr<class IAtlObjectInstance>	AtlObjectInstancesMgr;


#define ATL_MODEL_CLASS_NAME		"ModelObject"
#define ATL_ATL_OBJECT_CLASS_NAME	"AtlObject"
#define ATL_3D_PLANT_CLASS_NAME		"3DPlant"
#define ATL_BILLBOARD_CLASS_NAME	"Billboard"
#define	ATL_LIGHT_OBJECT_CLASS_NAME	"LightObject"


/** \interface IAtlObjectInstance 
 *	Class that represent object instance
 *
 * \author Abvent R&D Copyright (c) 2004
 * \author Richard Young
 * \version 1.0
 * \date 27/04/2004
 */
class IAtlObjectInstance
{
public:
	/// Definition of triangle id as opaque
	typedef const struct OpaqueTriangleId*	TriangleId;
	/// Definition of instance Id
	typedef tUUID			InstanceId;

	/// Put an XML representation of the object instances inside the father node.
	/** \retval the new node of the object.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the object instances with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(ConstXMLNodeRef inFatherNode) = 0;

	/// Return the name of the object.
	/** Do not delete this string. */
	virtual tUtf8					getName() const = 0;

	/// Set the name of the object.
	virtual void					setName(tUtf8 inName) = 0;

	/// Get the class of this object instance.
	virtual tUtf8					getClass() const = 0;

	/// Set the class of this object instance.
	virtual void					setClass(tUtf8 inClassName) = 0;

	/// Get the instance id.
	virtual tUUID					getInstanceId() const = 0;

	/// Set the instance id.
	virtual void					setInstanceId(tUUID inId) = 0;

	/// Get the instance id of the objet that we are attach too.
	/** Value of 0 mean no attachement. */
	virtual tUUID					getAttachId() const = 0;

	/// Set the instance id of the objet that we are attach too.
	/** Value of 0 mean no attachement. */
	virtual void					setAttachId(tUUID inId) = 0;

	//! Set the translation vector
	virtual void				setTranslation(const CAtlVector& inTranslation) = 0;
	//! Get the translation vector
	virtual const CAtlVector&	getTranslation() const = 0;

	//! Set the rotation vector
	virtual void				setRotation(const CAtlVector& inRotation) = 0;
	//! Get the rotation vector
	virtual const CAtlVector&	getRotation() const = 0;
	
	//! Set the scale vector
	virtual void				setScale(const CAtlVector& inScale) = 0;
	//! Get the scale vector
	virtual const CAtlVector&	getScale() const = 0;

	/// Get the triangle coordinates on which the instance is
	virtual const CAtlTriangle&		getTriangleCoord() const = 0;
	/// Set the triangle coordinates on which the instance is
	virtual void					setTriangleCoord(const CAtlTriangle& inTriangle) = 0;
	
	/// Get the id of the triangle that we are attach too.
	/** Value of 0 mean no triangle attachement */
	virtual TriangleId				getTriangleId() const = 0;

	/// Set the id of the triangle that we are attach too.
	/** Value of 0 mean no triangle attachement */
	virtual void					setTriangleId(TriangleId inId) = 0;

	/// Get the normal of the surface on which the instance is
	virtual const CAtlVector&		getNormal() const = 0;
	/// Set the normal of the surface on which the instance is
	virtual void					setNormal(const CAtlVector& inNormal) = 0;


	/// Get the anchor point of the definition
	virtual const CAtlVector&		getDefAnchorPoint() const = 0;
	/// Set the anchor point of the definition
	virtual void					setDefAnchorPoint(const CAtlVector& inPoint) = 0;

	/// Get the up vector of the definition
	virtual const CAtlVector&		getDefUpVector() const = 0;
	/// Set the up vector of the definition
	virtual void					setDefUpVector(const CAtlVector& inUpVector) = 0;

	/// Get the placement property of the definition
	virtual IAtlAtlObjectDefinition::eObjectPlacement	getDefPlacementProperty() const = 0;
	/// Set the placement property of the definition
	virtual void		setDefPlacementProperty(IAtlAtlObjectDefinition::eObjectPlacement inPlacementPropery) = 0;

	/// Get the initial angle of the definition
	virtual double			getDefInitialAngle() const = 0;
	/// Set the initial angle of the definition
	virtual void			setDefInitialAngle(double inAngle) = 0;

	/// Get the bounding box of the definition
	virtual const CAtlBoundingBox&	getDefBoundingBox() const = 0;
	/// Set the bounding box of the definition
	virtual void					setDefBoundingBox(const CAtlBoundingBox& inBB) = 0;

	/// Get the flag that lock the position and rotation of the object
	virtual bool					getCoordinatesLocked() const = 0;
	/// Set the flag that lock the position and rotation of the object
	virtual void					setCoordinatesLocked(bool inLock) = 0;

	/// Get the flag that lock the scale of the object
	virtual bool					getScaleLocked() const = 0;
	/// Set the flag that lock the scale of the object
	virtual void					setScaleLocked(bool inLock) = 0;

	/// Insure that the XML node ref contain up to date values
	virtual void 					synchronizeXML() const = 0;

	/// Get a non modifiable XML data of this instance.
	virtual ConstXMLNodeRef			getXMLNodeRef() const = 0;

	/// Get a modifiable XML data of this instance.
	virtual XMLNodeRef				getXMLNodeRef() = 0;

	/// Get user interfaces private value
	virtual int						getUIValue() const = 0;

	/// Set user interfaces private value
	virtual void					setUIValue(int inValue) = 0;

	/// Return the atl object instance.
	/** Return NULL if it's not a atl object */
	virtual const IAtlAtlObjectInstance*	asAtlObjectInstance() const = 0;

	/// Return the atl object instance.
	/** Return NULL if it's not a atl object */
	virtual IAtlAtlObjectInstance*			asAtlObjectInstance() = 0;

	/// Return the atl object instance.
	/** Return NULL if it's not a 3D plant */
	virtual const IAtl3DPlantRef*		as3DPlantInstance() const = 0;

	/// Return the atl object instance.
	/** Return NULL if it's not a 3D plant */
	virtual IAtl3DPlantRef*				as3DPlantInstance() = 0;

	/// Return the atl billboard object instance.
	/** Return NULL if it's not a billboard */
	virtual const IAtlBBObjectInstance*		asBBObjectInstance() const = 0;

	/// Return the atl billboard object instance.
	/** Return NULL if it's not a billboard */
	virtual IAtlBBObjectInstance*			asBBObjectInstance() = 0;

	/// Return the atl light object instance.
	/** Return NULL if it's not a light */
	virtual const IAtlObjectLightRef*		asLightObjectInstance() const = 0;

	/// Return the atl light object instance.
	/** Return NULL if it's not a light */
	virtual IAtlObjectLightRef*				asLightObjectInstance()  = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual							~IAtlObjectInstance() {}
};


} // namespace  AvAtl

#endif
