/** \file
 * Header for the interface IAtlParallelView
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Christophe Noblet
 * \version 1.0
 * \date 02/08/2001
 */

#ifndef IAtlParallelView_h
#define IAtlParallelView_h

#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif

#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif

#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif

namespace AvAtl{

// We force our enums to be int, so you dont have to change your project settings.
#ifdef __MWERKS__
#pragma enumsalwaysint on
#endif


/// Parallel view interface
/** \interface IAtlParallelView 
 *	Interface for setting parallel view.
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Christophe Noblet
 * \version 1.0
 * \date 02/08/2001
 */
class IAtlParallelView
{
public:
	/// Parallel view type for atl object
	typedef enum {
		ePVFront	= 0,	
		ePVPlan		= 1,
		ePVAxono	= 2
	} EPVTypes;	
	
	/// Get the name of the parallel view.
	virtual tUtf8					getName() const = 0;
	/// Set the name of the parallel view.
	virtual void					setName(tUtf8 name) = 0;
	
	/// Get the type of the parallel view.
	virtual EPVTypes				getType() const = 0;
	/// Set the type of the parallel view.
	virtual void 					setType(EPVTypes pvtype) = 0;

	/// Get the flags of the parallel view.
	/** Any combination of the AtlViewFlags constants. */
	virtual int						getFlags() const = 0;
	/// Set the flags of the parallel view.
	/** Any combination of the AtlViewFlags constants. */
	virtual void					setFlags(int flags) = 0;

	/// Get the plan view orientation.
	virtual double 					getNorth() const = 0;
	/// Set the plan view orientation.
	virtual void 					setNorth(double angle) = 0;

	/// Get the projection size in real space.
	virtual double					getSize() const = 0;
	/// Set the projection size in real space.
	virtual void					setSize(double projectionSize) = 0;

	/// Get the camera location in 3D space.
	virtual const CAtlVector&	 	getPosition() const = 0;
	/// Set the camera location in 3D space.
	virtual void 					setPosition(const CAtlVector& position) = 0;

	/// Get the target location in 3D space.
	virtual const CAtlVector& 		getTarget() const = 0;
	/// Set the target location in 3D space.
	virtual void 					setTarget(const CAtlVector& target) = 0;
 
// End of API 1

	/// Put an XML representation of the parallel view inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the parallel view with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(XMLNodeRef inFatherNode) = 0;

	/// Return the view info of this view.
	/** For consulting */
	virtual const class IAtlViewInfo*	getViewInfo() const = 0;
	/// Return the view info of this view.
	/** For setting */
	virtual class IAtlViewInfo*			getViewInfo() = 0;

	/// Return the options of the view.
	/** For consulting */
	virtual const class IAtlViewOpt*	getViewOpt() const = 0;
	/// Return the options of the view.
	/** For setting */
	virtual class IAtlViewOpt*			getViewOpt() = 0;

	/// Control the special true colors and 45 degres flag.
	virtual bool					getSpecialShadingFlag() const = 0;
	/// Control the special 45 degres flag.
	virtual void					setSpecialShadingFlag(bool inFlag) = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsability of the CAtlMgr class.*/
									~IAtlParallelView() {}
};


/// List of parallels views
typedef TAtlObjectMgr<IAtlParallelView>	AtlParallelViewMgr;


#ifdef __MWERKS__
#pragma enumsalwaysint reset
#endif
}
#endif